from teslajsonpy2.vehicle import VehicleDevice
import time


class Climate(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__is_auto_conditioning_on = False
        self.__inside_temp = 0
        self.__outside_temp = 0
        self.__driver_temp_setting = 0
        self.__passenger_temp_setting = 0
        self.__is_climate_on = False
        self.__fan_status = 0
        self.__manual_update_time = 0

        self.type = 'HVAC (climate) system'
        self.hass_type = 'climate'
        self.measurement = 'C'

        self.name = self._name()

        self.uniq_name = self._uniq_name()
        self.bin_type = 0x3

        self.update()

    def is_hvac_enabled(self):
        return self.__is_climate_on

    def get_current_temp(self):
        return self.__inside_temp

    def get_goal_temp(self):
        return self.__driver_temp_setting

    def get_fan_status(self):
        return self.__fan_status

    def update(self):
        self._controller.update(self._id)

        data = self._controller.get_climate_params(self._id)
        if data:
            if time.time() - self.__manual_update_time > 60:
                self.__is_auto_conditioning_on = data['is_auto_conditioning_on']
                self.__is_climate_on = data['is_climate_on']
                self.__driver_temp_setting = data['driver_temp_setting'] \
                    if data['driver_temp_setting'] else self.__driver_temp_setting
                self.__passenger_temp_setting = data['passenger_temp_setting'] \
                    if data['passenger_temp_setting'] else self.__passenger_temp_setting
            self.__inside_temp = data['inside_temp'] if data['inside_temp'] else self.__inside_temp
            self.__outside_temp = data['outside_temp'] if data['outside_temp'] else self.__outside_temp
            self.__fan_status = data['fan_status']

    def set_temperature(self, temp):
        temp = round(temp, 1)
        self.__manual_update_time = time.time()
        data = self._controller.command(self._id, 'set_temps', {"driver_temp": temp, "passenger_temp": temp})
        if data['response']['result']:
            self.__driver_temp_setting = temp
            self.__passenger_temp_setting = temp

    def set_status(self, enabled):
        self.__manual_update_time = time.time()
        if enabled:
            data = self._controller.command(self._id, 'auto_conditioning_start')
            if data['response']['result']:
                self.__is_auto_conditioning_on = True
                self.__is_climate_on = True
        else:
            data = self._controller.command(self._id, 'auto_conditioning_stop')
            if data['response']['result']:
                self.__is_auto_conditioning_on = False
                self.__is_climate_on = False
        self.update()

    @staticmethod
    def has_battery():
        return False


class TempSensor(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__inside_temp = 0
        self.__outside_temp = 0

        self.type = 'temperature sensor'
        self.measurement = 'C'
        self.hass_type = 'sensor'
        self.name = self._name()
        self.uniq_name = self._uniq_name()
        self.bin_type = 0x4
        self.update()

    def get_inside_temp(self):
        return self.__inside_temp

    def get_outside_temp(self):
        return self.__outside_temp

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_climate_params(self._id)
        if data:
            self.__inside_temp = data['inside_temp'] if data['inside_temp'] else self.__inside_temp
            self.__outside_temp = data['outside_temp'] if data['outside_temp'] else self.__outside_temp

    @staticmethod
    def has_battery():
        return False



from teslajsonpy2.vehicle import VehicleDevice


class GPS(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__longitude = 0
        self.__latitude = 0
        self.__heading = 0
        self.__location = {}

        self.last_seen = 0
        self.last_updated = 0
        self.type = 'location tracker'
        self.hass_type = 'devices_tracker'
        self.bin_type = 0x6

        self.name = self._name()

        self.uniq_name = self._uniq_name()
        self.update()

    def get_location(self):
        return self.__location

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_drive_params(self._id)
        if data:
            self.__longitude = data['longitude']
            self.__latitude = data['latitude']
            self.__heading = data['heading']
        if self.__longitude and self.__latitude and self.__heading:
            self.__location = {'longitude': self.__longitude,
                               'latitude': self.__latitude,
                               'heading': self.__heading}

    @staticmethod
    def has_battery():
        return False


class Odometer(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__odometer = 0
        self.type = 'mileage sensor'
        self.measurement = 'LENGTH_MILES'
        self.hass_type = 'sensor'
        self.name = self._name()
        self.uniq_name = self._uniq_name()
        self.bin_type = 0xB
        self.update()
        self.__rated = True

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_state_params(self._id)
        if data:
            self.__odometer = data['odometer']
        data = self._controller.get_gui_params(self._id)
        if data:
            if data['gui_distance_units'] == "mi/hr":
                self.measurement = 'LENGTH_MILES'
            else:
                self.measurement = 'LENGTH_KILOMETERS'
            self.__rated = (data['gui_range_display'] == "Rated")

    @staticmethod
    def has_battery():
        return False

    def get_value(self):
        return round(self.__odometer, 1)



from teslajsonpy2.vehicle import VehicleDevice
import time


class ChargerSwitch(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__manual_update_time = 0
        self.__charger_state = False
        self.type = 'charger switch'
        self.hass_type = 'switch'
        self.name = self._name()
        self.uniq_name = self._uniq_name()
        self.bin_type = 0x8
        self.update()

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_charging_params(self._id)
        if data and (time.time() - self.__manual_update_time > 60):
            if data['charging_state'] != "Charging":
                self.__charger_state = False
            else:
                self.__charger_state = True

    def start_charge(self):
        if not self.__charger_state:
            data = self._controller.command(self._id, 'charge_start')
            if data and data['response']['result']:
                self.__charger_state = True
            self.__manual_update_time = time.time()

    def stop_charge(self):
        if self.__charger_state:
            data = self._controller.command(self._id, 'charge_stop')
            if data and data['response']['result']:
                self.__charger_state = False
            self.__manual_update_time = time.time()

    def is_charging(self):
        return self.__charger_state

    @staticmethod
    def has_battery():
        return False


class RangeSwitch(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__manual_update_time = 0
        self.__maxrange_state = False
        self.type = 'maxrange switch'
        self.hass_type = 'switch'
        self.name = self._name()
        self.uniq_name = self._uniq_name()
        self.bin_type = 0x9
        self.update()

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_charging_params(self._id)
        if data and (time.time() - self.__manual_update_time > 60):
            self.__maxrange_state = data['charge_to_max_range']

    def set_max(self):
        if not self.__maxrange_state:
            data = self._controller.command(self._id, 'charge_max_range')
            if data['response']['result']:
                self.__maxrange_state = True
            self.__manual_update_time = time.time()

    def set_standard(self):
        if self.__maxrange_state:
            data = self._controller.command(self._id, 'charge_standard')
            if data and data['response']['result']:
                self.__maxrange_state = False
            self.__manual_update_time = time.time()

    def is_maxrange(self):
        return self.__maxrange_state

    @staticmethod
    def has_battery():
        return False



import time
from multiprocessing import RLock
from teslajsonpy2.connection import Connection
from teslajsonpy2.BatterySensor import Battery, Range
from teslajsonpy2.Lock import Lock
from teslajsonpy2.Climate import Climate, TempSensor
from teslajsonpy2.BinarySensor import ParkingSensor, ChargerConnectionSensor
from teslajsonpy2.Charger import ChargerSwitch, RangeSwitch
from teslajsonpy2.GPS import GPS, Odometer
from teslajsonpy2.Trunk import FrontTrunk, RearTrunk


class Controller:
    def __init__(self, email, password, update_interval):
        self.__connection = Connection(email, password)
        self.__vehicles = []
        self.update_interval = update_interval
        self.__climate = {}
        self.__charging = {}
        self.__state = {}
        self.__driving = {}
        self.__gui = {}
        self.__vehicle = {}
        self.__last_update_time = {}
        self.__update = {}
        self.__lock = RLock()
        cars = self.__connection.get('vehicles')['response']
        for car in cars:
            self.__last_update_time[car['id']] = 0
            self.__update[car['id']] = True
            self.update(car['id'])
            self.__vehicles.append(Climate(car, self))
            self.__vehicles.append(Battery(car, self))
            self.__vehicles.append(Range(car, self))
            self.__vehicles.append(TempSensor(car, self))
            self.__vehicles.append(Lock(car, self))
            self.__vehicles.append(ChargerConnectionSensor(car, self))
            self.__vehicles.append(ChargerSwitch(car, self))
            self.__vehicles.append(RangeSwitch(car, self))
            self.__vehicles.append(ParkingSensor(car, self))
            self.__vehicles.append(GPS(car, self))
            self.__vehicles.append(Odometer(car, self))
            self.__vehicles.append(FrontTrunk(car, self))
            self.__vehicles.append(RearTrunk(car, self))

    def post(self, vehicle_id, command, data={}):
        return self.__connection.post('vehicles/%i/%s' % (vehicle_id, command), data)

    def get(self, vehicle_id, command):
        return self.__connection.get('vehicles/%i/%s' % (vehicle_id, command))

    def data_request(self, vehicle_id, name):
        return self.get(vehicle_id, 'data_request/%s' % name)['response']

    def command(self, vehicle_id, name, data={}):
        return self.post(vehicle_id, 'command/%s' % name, data)

    def list_vehicles(self):
        return self.__vehicles

    def wake_up(self, vehicle_id):
        self.post(vehicle_id, 'wake_up')

    def update(self, car_id):
        cur_time = time.time()
        with self.__lock:
            if (self.__update[car_id] and (cur_time - self.__last_update_time[car_id] > self.update_interval)):
                self.wake_up(car_id)
                data = self.get(car_id, 'data')
                if data and data['response']:
                    self.__climate[car_id] = data['response']['climate_state']
                    self.__charging[car_id] = data['response']['charge_state']
                    self.__state[car_id] = data['response']['vehicle_state']
                    self.__driving[car_id] = data['response']['drive_state']
                    self.__gui[car_id] = data['response']['gui_settings']
                    self.__last_update_time[car_id] = time.time()
                    self.__vehicle[car_id] = data['response']['vehicle_state']
                else:
                    self.__climate[car_id] = False
                    self.__charging[car_id] = False
                    self.__state[car_id] = False
                    self.__driving[car_id] = False
                    self.__gui[car_id] = False
                    self.__vehicle[car_id] = False
    def get_climate_params(self, car_id):
        return self.__climate[car_id]

    def get_charging_params(self, car_id):
        return self.__charging[car_id]

    def get_state_params(self, car_id):
        return self.__state[car_id]

    def get_drive_params(self, car_id):
        return self.__driving[car_id]

    def get_gui_params(self, car_id):
        return self.__gui[car_id]

    def get_vehicle_params(self, car_id):
        return self.__vehicle[car_id]

    def get_updates(self, car_id=None):
        if car_id is not None:
            return self.__update[car_id]
        else:
            return self.__update

    def set_updates(self, car_id, value):
        self.__update[car_id] = value


class VehicleDevice:
    def __init__(self, data, controller):
        self._id = data['id']
        self._vehicle_id = data['vehicle_id']
        self._vin = data['vin']
        self._state = data['state']
        self._controller = controller
        self.should_poll = True

    def _name(self):
        return 'Tesla Model {} {}'.format(
            str(self._vin[3]).upper(), self.type)

    def _uniq_name(self):
        return 'Tesla Model {} {} {}'.format(
            str(self._vin[3]).upper(), self._vin, self.type)
    
    def id(self):
        return self._id 
    
    @staticmethod
    def is_armable():
        return False

    @staticmethod
    def is_armed():
        return False


from teslajsonpy2.vehicle import VehicleDevice
import time


class FrontTrunk(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__manual_update_time = 0
        self.__front_trunk_state = False
        self.type = 'front trunk lock'
        self.hass_type = 'lock'
        self.name = self._name()
        self.uniq_name = self._uniq_name()
        self.bin_type = 0xC
        self.update()

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_vehicle_params(self._id)
        if data and (time.time() - self.__manual_update_time > 60):
            self.__front_trunk_state = data['ft']

    def open_front_trunk(self):
        if not self.__front_trunk_state:
            data = self._controller.command(self._id, 'trunk_open', {"which_trunk": 'front'})
            if data and data['response']['result']:
                self.__front_trunk_state = True
            self.__manual_update_time = time.time()

    def is_locked(self):
        return not self.__front_trunk_state

    @staticmethod
    def has_battery():
        return False

class RearTrunk(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__manual_update_time = 0
        self.__rear_trunk_state = False
        self.type = 'rear trunk lock'
        self.hass_type = 'lock'
        self.name = self._name()
        self.uniq_name = self._uniq_name()
        self.bin_type = 0xD
        self.update()

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_vehicle_params(self._id)
        if data and (time.time() - self.__manual_update_time > 60):
            self.__rear_trunk_state = data['rt']

    def open_rear_trunk(self):
        if not self.__rear_trunk_state:
            data = self._controller.command(self._id, 'trunk_open', {"which_trunk": 'rear'})
            if data and data['response']['result']:
                self.__rear_trunk_state = True
            self.__manual_update_time = time.time()

    def is_locked(self):
        return not self.__rear_trunk_state

    @staticmethod
    def has_battery():
        return False


from teslajsonpy2.BatterySensor import (Battery, Range)
from teslajsonpy2.BinarySensor import (ChargerConnectionSensor, ParkingSensor)
from teslajsonpy2.Charger import (ChargerSwitch, RangeSwitch)
from teslajsonpy2.Climate import (Climate, TempSensor)
from teslajsonpy2.Trunk import (FrontTrunk, RearTrunk)
from teslajsonpy2.controller import Controller
from teslajsonpy2.Exceptions import TeslaException
from teslajsonpy2.GPS import GPS, Odometer
from teslajsonpy2.Lock import Lock




from teslajsonpy2.vehicle import VehicleDevice


class ParkingSensor(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__state = False

        self.type = 'parking brake sensor'
        self.hass_type = 'binary_sensor'

        self.name = self._name()

        self.uniq_name = self._uniq_name()
        self.bin_type = 0x1
        self.update()

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_drive_params(self._id)
        if data:
            if not data['shift_state'] or data['shift_state'] == 'P':
                self.__state = True
            else:
                self.__state = False

    def get_value(self):
        return self.__state

    @staticmethod
    def has_battery():
        return False


class ChargerConnectionSensor(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__state = False

        self.type = 'charger sensor'
        self.hass_type = 'binary_sensor'
        self.name = self._name()

        self.uniq_name = self._uniq_name()
        self.bin_type = 0x2

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_charging_params(self._id)
        if data:
            if data['charging_state'] in ["Disconnected", "Stopped", "NoPower"]:
                self.__state = False
            else:
                self.__state = True

    def get_value(self):
        return self.__state

    @staticmethod
    def has_battery():
        return False



class TeslaException(Exception):
    def __init__(self, code,  *args, **kwargs):
        self.message = ""
        super().__init__(*args, **kwargs)
        self.code = code
        if self.code == 401:
            self.message = 'UNAUTHORIZED'
        elif self.code == 404:
            self.message = 'NOT_FOUND'
        elif self.code == 405:
            self.message = 'MOBILE_ACCESS_DISABLED'
        elif self.code == 423:
            self.message = 'ACCOUNT_LOCKED'
        elif self.code == 429:
            self.message = 'TOO_MANY_REQUESTS'
        elif self.code == 500:
            self.message = 'SERVER_ERROR'
        elif self.code == 503:
            self.message = 'SERVICE_MAINTENANCE'
        elif self.code > 299:
            self.message = "UNKNOWN_ERROR"



import calendar
import datetime
from urllib.parse import urlencode
from urllib.request import Request, build_opener
from urllib.error import HTTPError
import json
from teslajsonpy2.Exceptions import TeslaException


class Connection(object):
    """Connection to Tesla Motors API"""
    def __init__(self, email, password):
        """Initialize connection object"""
        self.user_agent = 'Model S 2.1.79 (SM-G900V; Android REL 4.4.4; en_US'
        self.client_id = "81527cff06843c8634fdc09e8ac0abefb46ac849f38fe1e431c2ef2106796384"
        self.client_secret = "c7257eb71a564034f9419ee651c7d0e5f7aa6bfbd18bafb5c5c033b093bb2fa3"
        self.baseurl = 'https://owner-api.teslamotors.com'
        self.api = '/api/1/'
        self.oauth = {
            "grant_type": "password",
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "email": email,
            "password": password}
        self.expiration = 0

    def get(self, command):
        """Utility command to get data from API"""
        return self.post(command, None)

    def post(self, command, data={}):
        """Utility command to post data to API"""
        now = calendar.timegm(datetime.datetime.now().timetuple())
        if now > self.expiration:
            auth = self.__open("/oauth/token", data=self.oauth)
            self.__sethead(auth['access_token'])
        return self.__open("%s%s" % (self.api, command), headers=self.head, data=data)

    def __sethead(self, access_token):
        """Set HTTP header"""
        self.access_token = access_token
        now = calendar.timegm(datetime.datetime.now().timetuple())
        self.expiration = now + 1800
        self.head = {"Authorization": "Bearer %s" % access_token,
                     "User-Agent": self.user_agent
                     }

    def __open(self, url, headers={}, data=None, baseurl=""):
        """Raw urlopen command"""
        if not baseurl:
            baseurl = self.baseurl
        req = Request("%s%s" % (baseurl, url), headers=headers)
        try:
            req.data = urlencode(data).encode('utf-8')
        except TypeError:
            pass
        opener = build_opener()

        try:
            resp = opener.open(req)
            charset = resp.info().get('charset', 'utf-8')
            data = json.loads(resp.read().decode(charset))
            opener.close()
            return data
        except HTTPError as e:
            if e.code == 408:
                return False
            else:
                raise TeslaException(e.code)



from teslajsonpy2.vehicle import VehicleDevice
import time


class Lock(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__manual_update_time = 0
        self.__lock_state = False

        self.type = 'door lock'
        self.hass_type = 'lock'

        self.name = self._name()

        self.uniq_name = self._uniq_name()
        self.bin_type = 0x7
        self.update()

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_state_params(self._id)
        if data and (time.time() - self.__manual_update_time > 60):
            self.__lock_state = data['locked']

    def lock(self):
        if not self.__lock_state:
            data = self._controller.command(self._id, 'door_lock')
            if data['response']['result']:
                self.__lock_state = True
            self.__manual_update_time = time.time()

    def unlock(self):
        if self.__lock_state:
            data = self._controller.command(self._id, 'door_unlock')
            if data['response']['result']:
                self.__lock_state = False
            self.__manual_update_time = time.time()

    def is_locked(self):
        return self.__lock_state

    @staticmethod
    def has_battery():
        return False



from teslajsonpy2.vehicle import VehicleDevice


class Battery(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__battery_level = 0
        self.__charging_state = None
        self.__charge_port_door_open = None
        self.type = 'battery sensor'
        self.measurement = '%'
        self.hass_type = 'sensor'
        self.name = self._name()
        self.uniq_name = self._uniq_name()
        self.bin_type = 0x5
        self.update()

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_charging_params(self._id)
        if data:
            self.__battery_level = data['battery_level']
            self.__charging_state = data['charging_state']

    @staticmethod
    def has_battery():
        return False

    def get_value(self):
        return self.__battery_level


class Range(VehicleDevice):
    def __init__(self, data, controller):
        super().__init__(data, controller)
        self.__battery_range = 0
        self.__est_battery_range = 0
        self.__ideal_battery_range = 0
        self.type = 'range sensor'
        self.__rated = True
        self.measurement = 'LENGTH_MILES'
        self.hass_type = 'sensor'
        self.name = self._name()
        self.uniq_name = self._uniq_name()
        self.bin_type = 0xA
        self.update()

    def update(self):
        self._controller.update(self._id)
        data = self._controller.get_charging_params(self._id)
        if data:
            self.__battery_range = data['battery_range']
            self.__est_battery_range = data['est_battery_range']
            self.__ideal_battery_range = data['ideal_battery_range']
        data = self._controller.get_gui_params(self._id)
        if data:
            if data['gui_distance_units'] == "mi/hr":
                self.measurement = 'LENGTH_MILES'
            else:
                self.measurement = 'LENGTH_KILOMETERS'
            self.__rated = (data['gui_range_display'] == "Rated")

    @staticmethod
    def has_battery():
        return False

    def get_value(self):
        if self.__rated:
            return self.__battery_range
        else:
            return self.__ideal_battery_range



from setuptools import setup
setup(
    name='teslajsonpy2',
    version='0.1.24',
    packages=['teslajsonpy2'],
    include_package_data=True,
    python_requires='>=3',
    license='WTFPL',
    description='A library to work with Tesla API.',
    long_description='A library to work with Tesla car API.',
    url='https://github.com/rudical/teslajsonpy2',
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet',
    ],
)
